@@ -3,8 +3,8 @@
 ;; Copyright (C) 2024-2026 James Cherti | https://www.jamescherti.com/contact/
 
 ;; Author: James Cherti <https://www.jamescherti.com/contact/>
-;; Package-Version: 20260215.2153
-;; Package-Revision: 4de82266d54b
+;; Package-Version: 20260216.304
+;; Package-Revision: 29308eb50800
 ;; URL: https://github.com/jamescherti/compile-angel.el
 ;; Keywords: convenience
 ;; Package-Requires: ((emacs "26.3"))
@@ -24,40 +24,23 @@
 ;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
-;; The compile-angel package automatically byte-compiles and native-compiles
-;; Emacs Lisp libraries. It offers:
-;; - (compile-angel-on-load-mode): A global mode that compiles .el files before
-;;   they are loaded.
-;; - (compile-angel-on-save-local-mode): A local mode that compiles .el files
+;; The compile-angel package speeds up Emacs by ensuring that all Elisp
+;; libraries are both byte-compiled and native-compiled:
+;; - Byte compilation reduces the overhead of loading Emacs Lisp code at
+;;   runtime.
+;; - Native compilation improves performance by generating machine code that
+;;   runs directly on the hardware, leveraging the full capabilities of the host
+;;   CPU. The actual speedup varies with the characteristics of the Lisp code,
+;;   but it is typically 2.5 to 5 times faster than the equivalent byte-compiled
+;;   version.
+;;
+;; This package offers:
+;; - `compile-angel-on-load-mode': A global mode that compiles .el files both
+;;   before they are loaded via `load' or `require', and after they are loaded,
+;;   using `after-load-functions'.
+;; - `compile-angel-on-save-local-mode': A local mode that compiles .el files
 ;;   whenever the user saves them.
 ;;
-;; The compile-angel modes speed up Emacs by ensuring all libraries are
-;; byte-compiled and native-compiled. Byte-compilation reduces the overhead of
-;; loading Emacs Lisp code at runtime, while native compilation optimizes
-;; performance by generating machine code specific to your system.
-;;
-;; The author of compile-angel was previously a user of auto-compile but
-;; encountered an issue where several .el files were not being compiled by
-;; auto-compile, resulting in Emacs performance degradation due to the lack of
-;; native compilation. After extensive experimentation and research, the author
-;; developed compile-angel to address this problem. The compile-angel package
-;; guarantees that all .el files are both byte-compiled and native-compiled,
-;; which significantly speeds up Emacs.
-;;
-;; The compile-angel package was created to offer an alternative to auto-compile
-;; that guarantees all .el files are both byte-compiled and native-compiled,
-;; which significantly speeds up Emacs.
-;;
-;; Before installing:
-;; ------------------
-;; It is highly recommended to set the following variables in your init file:
-;;   (setq load-prefer-newer t)
-;;   (setq native-comp-jit-compilation t)
-;;   (setq native-comp-deferred-compilation t) ; Deprecated in Emacs > 29.1
-;;
-;; Additionally, ensure that native compilation is enabled; this should
-;; return t: `(native-comp-available-p)`.
-;;
 ;; Installation from MELPA:
 ;; ------------------------
 ;; (use-package compile-angel
@@ -74,9 +57,19 @@
 ;;   ;; Uncomment the line below to auto compile when an .el file is saved
 ;;   ;; (add-hook 'emacs-lisp-mode-hook #'compile-angel-on-save-local-mode)
 ;;
+;;   ;; The following directive prevents compile-angel from compiling your init
+;;   ;; files. If you choose to remove this push to
+;;   ;; `compile-angel-excluded-files' and compile your pre/post-init files,
+;;   ;; ensure you understand the implications and thoroughly test your code.
+;;   ;; For example, if you're using the `use-package' macro, you'll need to
+;;   ;; explicitly add: (eval-when-compile (require 'use-package)) at the top of
+;;   ;; your init file.
+;;   (push "/init.el" compile-angel-excluded-files)
+;;   (push "/early-init.el" compile-angel-excluded-files)
+;;
 ;;   ;; A global mode that compiles .el files before they are loaded
 ;;   ;; using `load' or `require'.
-;;   (compile-angel-on-load-mode))
+;;   (compile-angel-on-load-mode 1))
 ;;
 ;; Links:
 ;; ------
@@ -87,7 +80,14 @@
 
 (require 'bytecomp)
 (require 'cl-lib)
-(eval-when-compile (require 'subr-x))
+
+;; This is for Emacs 28 or older. Starting with Emacs 29.1,
+;; `string-remove-suffix' was moved to the core, making it available
+;; automatically without requiring extra libraries.
+;;
+;; The `subr-x' library provides utility functions like `string-remove-suffix'
+;; that are invoked at runtime within `compile-angel--update-el-file-regexp'.
+(require 'subr-x)
 
 ;;; Variables
 
@@ -106,16 +106,20 @@
   :type 'boolean
   :group 'compile-angel)
 
-(defcustom compile-angel-excluded-files '("loaddefs.el"
-                                          "autoloads.el"
-                                          "/lisp/org/org-version.el"
-                                          "/lisp/cus-load.el"
-                                          "/lisp/finder-inf.el"
-                                          "/.yas-setup.el"  ; Yasnippet
-                                          ;; /lisp and /site-lisp: subdirs.el
-                                          "lisp/subdirs.el"
-                                          ;; Built-in no-byte-compile packages
-                                          "/lisp/leim/leim-list.el")
+(defcustom compile-angel-excluded-files
+  (delq nil (list "-loaddefs.el"
+                  "-autoloads.el"
+                  "-pkg.el"          ; Package metadata
+                  "/loaddefs.el"
+                  "/.dir-locals.el"
+                  "/lisp/org/org-version.el"
+                  "/lisp/cus-load.el"
+                  "/lisp/finder-inf.el"
+                  "/.yas-setup.el"  ; Yasnippet
+                  ;; /lisp and /site-lisp: subdirs.el
+                  "lisp/subdirs.el"
+                  ;; Built-in no-byte-compile packages
+                  "/lisp/leim/leim-list.el"))
   "A list of path suffixes used to exclude specific .el files from compilation.
 
 Example: \\='(\"suffix.el\" \"/filename.el\") This excludes any path that ends
@@ -135,7 +139,34 @@ include all extensions associated with .el files."
 (defcustom compile-angel-excluded-files-regexps
   (delq nil (list "/lisp/international/.*\\.el"
                   (when (bound-and-true-p doom-user-dir)
-                    "/doom-snippets/.*")))
+                    "/doom-snippets/.*")
+
+                  ;; Spacemacs: init.el and early-init.el
+                  (when (bound-and-true-p spacemacs-core-directory)
+                    (concat "^" (regexp-quote (expand-file-name
+                                               "init.el"
+                                               user-emacs-directory)) "$"))
+                  (when (bound-and-true-p spacemacs-core-directory)
+                    (concat "^" (regexp-quote (expand-file-name
+                                               "early-init.el"
+                                               user-emacs-directory)) "$"))
+
+                  ;; Spacemacs: Exclude Spacemacs core directory
+                  (when (bound-and-true-p spacemacs-core-directory)
+                    (concat "^" (regexp-quote (file-name-as-directory
+                                               (expand-file-name
+                                                spacemacs-core-directory)))))
+
+                  ;; Spacemacs: Files such as ~/.spacemacs
+                  (when (bound-and-true-p dotspacemacs-filepath)
+                    (concat "^" (regexp-quote (expand-file-name
+                                               dotspacemacs-filepath)) "$"))
+
+                  ;; Spacemacs: Exclude layers directory only if core exists
+                  (when (bound-and-true-p spacemacs-core-directory)
+                    (concat "^" (regexp-quote (expand-file-name
+                                               "layers/"
+                                               user-emacs-directory))))))
   "A list of regular expressions to exclude certain .el files from compilation.
 
 It is advisable to use `compile-angel-excluded-files' instead of
@@ -233,9 +264,27 @@ The value determines how file name handlers are temporarily adjusted:
 
 - \\='all   Disable all handlers (NOT RECOMMENDED).
 - \\='safe  Disable all handlers except the compression handler
-  (`.el.gz' files).
+           (`.el.gz' files).
 - \\='nil   Do not modify `file-name-handler-alist'.
 
+Every time Emacs performs a file operation (like `load', `file-exists-p',
+`expand-file-name'), it iterates through `file-name-handler-alist':
+- It checks the file path against every regular expression in that
+  list.
+- This list often contains complex regexes for TRAMP (remote files), image
+  libraries, archives, and encryption.
+- By setting it to 'safe (which usually leaves only one or zero entries), you
+  skip all those regex checks. In a loop compiling hundreds of files, saving
+  ~0.5ms per file operation adds up to noticeable seconds.
+
+For startup optimization: Yes. Many Emacs style configs use this trick during
+startup because they know exactly which files are being loaded (usually local,
+unencrypted ones).
+
+For general purpose compilation: It depends.
+- If you only compile local files: Use 'safe. It gives you free performance.
+- If you use TRAMP: You must use 'nil.
+
 This setting is intended to improve performance by reducing the overhead
 associated with file-related operations during compilation."
   :type '(choice (const :tag "Disable all (NOT RECOMMENDED)" all)
@@ -243,6 +292,22 @@ associated with file-related operations during compilation."
                  (const :tag "No optimization" nil))
   :group 'compile-angel)
 
+(defcustom compile-angel-exclude-core-emacs-directory nil
+  "Non-nil to exclude the Emacs installation Lisp directory.
+This prevents the compilation of core Emacs files found in the directory
+containing `simple.el`."
+  :type 'boolean
+  :group 'compile-angel)
+
+(defvar compile-angel--lisp-directory
+  ;; Directory where Emacs's own *.el and *.elc Lisp files are installed.
+  (if (bound-and-true-p lisp-directory)
+      ;; Always use `file-truename'
+      (file-truename lisp-directory)
+    (when-let* ((library-path (locate-library "simple")))
+      ;; Always use `file-truename'
+      (file-truename (file-name-directory (file-truename library-path))))))
+
 ;;; Experimental features
 
 (defvar compile-angel-guess-el-file-use-load-history nil)
@@ -251,6 +316,12 @@ associated with file-related operations during compilation."
 When non-nil, construct a hash table mapping feature names to their file paths
 by scanning all directories in `load-path' to improve lookup performance.")
 
+(defvar compile-angel-native-compile-load nil
+  "Experimental feature. Do not activate it.
+Non-nil to load the file after asynchronous native compilation.
+When non-nil, passing the LOAD argument to `native-compile-async' ensures that
+the file is loaded immediately after compilation finishes.")
+
 (defvar compile-angel-reload-compiled-version nil
   "Experimental feature. Do not activate it.")
 
@@ -294,10 +365,12 @@ don't have associated .el files and therefore don't need compilation.")
 (defvar compile-angel--native-compile-when-jit-enabled nil)
 (defvar compile-angel--reload-after-native-compile (make-hash-table :test 'equal))
 
-(defvar compile-angel--list-compiled-files (make-hash-table :test 'equal))
+(defvar compile-angel--list-processed-files (make-hash-table :test 'equal))
 (defvar compile-angel--list-compiled-features (make-hash-table :test 'eq))
 
 (defvar compile-angel--legacy-currently-compiling nil)
+
+(defvar compile-angel--currently-compiling-use-hash nil)
 (defvar compile-angel--currently-compiling-files (make-hash-table :test 'equal))
 (defvar compile-angel--currently-compiling-features (make-hash-table :test 'eq))
 
@@ -403,7 +476,13 @@ declaration is absent or not trusted under safe-local-variable rules."
 
           ;; Disable any influence from .dir-locals.el files in the directory.
           ;; Ensures only variables from FILE-PATH are considered.
-          (enable-dir-local-variables nil)
+          ;;
+          ;; This prevents projects from disabling byte-compilation for an
+          ;; entire directory via .dir-locals.el, which is a standard Emacs
+          ;; mechanism.
+          ;;
+          ;; Do not enable this.
+          ;; (enable-dir-local-variables nil)
 
           ;; Disable remote directory-local variables via TRAMP or similar.
           ;; Prevents remote configuration from influencing results.
@@ -445,7 +524,7 @@ Return non-nil if the file should be ignored, nil otherwise."
   "Return non-nil if EL-FILE is native-compiled and up to date.
 Return nil if it is not native-compiled or if its .eln file is out of date."
   (let ((eln-file (and (fboundp 'comp-el-to-eln-filename)
-                       (funcall 'comp-el-to-eln-filename el-file))))
+                       (comp-el-to-eln-filename el-file))))
     (when (and eln-file
                (file-newer-than-file-p eln-file el-file))
       t)))
@@ -467,11 +546,15 @@ Return nil if it is not native-compiled or if its .eln file is out of date."
         (compile-angel--verbose-message
           "Async native-compilation: %s" el-file-abbreviated))
       (let ((inhibit-message (not (or (not compile-angel-verbose)
-                                      (not compile-angel-debug)))))
+                                      (not compile-angel-debug))))
+            ;; TODO find a way to pass the truename to this function
+            (el-file-truename (file-truename el-file)))
         (when (fboundp 'native-compile-async)
           (when compile-angel-reload-compiled-version
-            (puthash el-file t compile-angel--reload-after-native-compile))
-          (funcall 'native-compile-async el-file)))))))
+            (puthash el-file t
+                     compile-angel--reload-after-native-compile))
+          (native-compile-async el-file-truename nil
+                                compile-angel-native-compile-load)))))))
 
 (defun compile-angel--byte-compile (el-file elc-file)
   "Byte-compile EL-FILE into ELC-FILE.
@@ -495,46 +578,29 @@ Return the byte compile result."
            (emacs-lisp-mode-hook nil)
            (byte-compile-result
             (let ((original-message (symbol-function 'message)))
-              (cl-letf (((symbol-function #'message)
-                         #'(lambda (format-string &rest messages-args)
-                             (let ((combined-args (cons format-string
-                                                        messages-args)))
-                               (when (or (not compile-angel--quiet-byte-compile)
-                                         ;; Ensure format-string is a string
-                                         ;; before checking prefixes. This
-                                         ;; prevents crashes on (message nil).
-                                         (and compile-angel--quiet-byte-compile
-                                              (stringp format-string)
-                                              (not (string-prefix-p
-                                                    "Wrote"
-                                                    format-string))
-                                              (not (string-prefix-p
-                                                    "Compiling "
-                                                    format-string))))
-                                 ;; Show the message
-                                 (apply original-message combined-args))))))
-                (condition-case err
-                    (let ((noninteractive t))
-                      (byte-compile-file el-file))
-                  ;; TODO ignore all errors
-                  (permission-denied
-                   (progn
-                     (compile-angel--debug-message
-                       "Byte-compilation ignored: Permission denied: %s"
-                       (error-message-string err))
-                     ;; Try to native compile
-                     'byte-compile-exception-error))
-
-                  (t
+              (condition-case err
+                  (byte-compile-file el-file)
+                (permission-denied
+                 (progn
                    (compile-angel--debug-message
-                     "Byte-compilation ignored: Error: %s"
+                     "Byte-compilation ignored: Permission denied: %s"
                      (error-message-string err))
-                   'byte-compile-exception-error))))))
+                   ;; Try to native compile
+                   'byte-compile-exception-error))
+
+                (t
+                 (compile-angel--debug-message
+                   "Byte-compilation ignored: Error: %s"
+                   (error-message-string err))
+                 ;; Do not native compile
+                 nil)))))
       byte-compile-result))))
 
-(defun compile-angel--need-compilation-p (el-file feature)
+(defun compile-angel--need-compilation-p (el-file el-file-truename feature)
   "Return non-nil if EL-FILE or FEATURE need compilation.
 EL-FILE is a String representing the path to the Elisp source file.
+EL-FILE-TRUENAME is a `file-truename' String representing the path to the Elisp
+source file.
 FEATURE is a symbol representing the feature being loaded."
   (cond
    ((not el-file)
@@ -578,22 +644,43 @@ FEATURE is a symbol representing the feature being loaded."
        ((eq decision :force-compile)
         t)
 
+       ;; Exclude features ending with -autoloads
+       ((and feature
+             (or (eq feature 'loaddefs)
+                 (let ((feature (symbol-name feature)))
+                   (or (string-suffix-p "-autoloads" feature)
+                       (string-suffix-p "-pkg" feature)
+                       (string-suffix-p "-loaddefs" feature)))))
+        (compile-angel--debug-message
+          "SKIP (Feature ends with -autoloads/-pkg/-loaddefs): %s | %s"
+          el-file feature)
+        nil)
+
+       ;; Exclude Emacs Lisp Directory
+       ((and compile-angel-exclude-core-emacs-directory
+             compile-angel--lisp-directory
+             (string-prefix-p compile-angel--lisp-directory
+                              el-file-truename))
+        (compile-angel--debug-message
+          "SKIP (Emacs Lisp directory): %s | %s"
+          el-file feature)
+        nil)
+
        ;; This is specific to Doom Emacs: It ensures that the Doom Emacs user
        ;; directory, Emacs directory, and modules directory are not compiled by
        ;; compile-angel. This is important because `.el` files in these
        ;; directories should never be compiled, or Doom may fail to load some of
        ;; them correctly.
        ((and compile-angel--doom-user-dir
-             (let ((el-file-truename (file-truename el-file)))
-               (or (and compile-angel--doom-user-dir
-                        (string-prefix-p compile-angel--doom-user-dir
-                                         el-file-truename))
-                   (and compile-angel--doom-emacs-lisp-dir
-                        (string-prefix-p compile-angel--doom-emacs-lisp-dir
-                                         el-file-truename))
-                   (and compile-angel--doom-modules-dir
-                        (string-prefix-p compile-angel--doom-modules-dir
-                                         el-file-truename)))))
+             (or (and compile-angel--doom-user-dir
+                      (string-prefix-p compile-angel--doom-user-dir
+                                       el-file-truename))
+                 (and compile-angel--doom-emacs-lisp-dir
+                      (string-prefix-p compile-angel--doom-emacs-lisp-dir
+                                       el-file-truename))
+                 (and compile-angel--doom-modules-dir
+                      (string-prefix-p compile-angel--doom-modules-dir
+                                       el-file-truename))))
         (compile-angel--debug-message
           "SKIP (Doom Emacs modules/emacs/user directory): %s | %s"
           el-file feature)
@@ -614,6 +701,7 @@ FEATURE is a symbol representing the feature being loaded."
         nil)
 
        ;; :continue starts here:
+       ;; TODO add another one for file-truename?
        ((compile-angel--el-file-excluded-p el-file)
         nil)
 
@@ -629,6 +717,8 @@ When NOERROR is non-nil, suppress warnings if the file is absent."
                                 #'byte-compile-dest-file)
                               el-file))
            (do-native-compile nil)
+           (el-file-truename (when el-file
+                               (file-truename el-file)))
            (compile-angel--native-compile-when-jit-enabled
             compile-angel--native-compile-when-jit-enabled)
            no-byte-compile-defined)
@@ -654,7 +744,8 @@ When NOERROR is non-nil, suppress warnings if the file is absent."
             (if (compile-angel--no-byte-compile-p el-file)
                 (progn
                   (setq do-native-compile nil)
-                  (puthash el-file t compile-angel--no-byte-compile-files-list)
+                  (puthash el-file-truename t
+                           compile-angel--no-byte-compile-files-list)
                   (setq no-byte-compile-defined t)
                   (compile-angel--debug-message
                     "Native-compilation ignored (no-byte-compile): %s"
@@ -684,7 +775,8 @@ When NOERROR is non-nil, suppress warnings if the file is absent."
               (setq do-native-compile t))
 
              ((eq byte-compile-result 'no-byte-compile)
-              (puthash el-file t compile-angel--no-byte-compile-files-list)
+              (puthash el-file-truename t
+                       compile-angel--no-byte-compile-files-list)
               (setq no-byte-compile-defined t)
               (compile-angel--debug-message
                 "Byte-compilation Ignore (no-byte-compile): %s"
@@ -1003,50 +1095,77 @@ resolved file path or nil if not found."
         feature-file)))))
 
 (defun compile-angel--entry-point (el-file &optional feature nosuffix noerror)
-  "This function is called by all the :before advices.
-EL-FILE, FEATURE, NOERROR, and NOSUFFIX are the same arguments as
-`load' and `require'."
+  "This function is called by all the :before advices."
   (compile-angel--with-fast-file-ops
     (when (or compile-angel-enable-byte-compile
               compile-angel-enable-native-compile)
       (let* ((feature-symbol (compile-angel--normalize-feature feature))
              (el-file (compile-angel--guess-el-file
-                       el-file feature-symbol nosuffix)))
+                       el-file feature-symbol nosuffix))
+             ;; FIX: Canonicalize path for locking mechanisms
+             (el-file-truename (when el-file (file-truename el-file))))
         (cond
          ((not el-file)
           (compile-angel--debug-message
             "SKIP file (Returned a nil .el file): %s | %s" el-file feature))
 
-         ((member el-file compile-angel--legacy-currently-compiling)
+         ((member el-file-truename compile-angel--legacy-currently-compiling)
           (compile-angel--debug-message
             "SKIP file - LEGACY (To prevent recursive compilation): %s | %s"
             el-file feature))
 
-         ((and feature-symbol
+         ((and compile-angel--currently-compiling-use-hash
+               feature-symbol
                (gethash feature-symbol compile-angel--currently-compiling-features))
           (compile-angel--debug-message
             "SKIP feature (To prevent recursive compilation): %s | %s"
             el-file feature))
 
-         ((gethash el-file compile-angel--currently-compiling-files)
+         ((and compile-angel--currently-compiling-use-hash
+               (gethash el-file-truename compile-angel--currently-compiling-files))
           (compile-angel--debug-message
             "SKIP file (To prevent recursive compilation): %s | %s"
             el-file feature))
 
+         ;; Optimization: Check if we have already processed this file (compiled
+         ;; OR skipped) We treat `compile-angel--list-processed-files' as a
+         ;; "processed" cache.
          ((and compile-angel-on-load-mode-compile-once
-               (or (gethash el-file compile-angel--list-compiled-files)
+               (or (gethash el-file-truename compile-angel--list-processed-files)
                    (when feature-symbol
                      (gethash feature-symbol
                               compile-angel--list-compiled-features))))
           (compile-angel--debug-message
             "SKIP (In the skip hash list): %s | %s" el-file feature))
 
-         ((not (compile-angel--need-compilation-p el-file feature-symbol))
+         ((not (compile-angel--need-compilation-p el-file
+                                                  el-file-truename
+                                                  feature-symbol))
+          ;; Optimization: Negative Caching
+          ;; We're recording the "Skip" decision in future requests for the same
+          ;; file will trigger the fast-path check at the top of your cond
+          ;; block. This prevents compile-angel from re-running the expensive
+          ;; checks
+          (when compile-angel-on-load-mode-compile-once
+            (puthash el-file-truename t compile-angel--list-processed-files)
+            (when feature-symbol
+              (puthash feature-symbol t compile-angel--list-compiled-features)))
+
           (compile-angel--debug-message
             "SKIP (Does not need compilation): %s | %s" el-file feature))
 
          ((and compile-angel--track-no-byte-compile-files
-               (gethash el-file compile-angel--no-byte-compile-files-list))
+               (gethash el-file-truename compile-angel--no-byte-compile-files-list))
+          ;; Optimization: Negative Caching
+          ;; We're recording the "Skip" decision in future requests for the same
+          ;; file will trigger the fast-path check at the top of your cond
+          ;; block. This prevents compile-angel from re-running the expensive
+          ;; checks
+          (when compile-angel-on-load-mode-compile-once
+            (puthash el-file-truename t compile-angel--list-processed-files)
+            (when feature-symbol
+              (puthash feature-symbol t compile-angel--list-compiled-features)))
+
           (compile-angel--debug-message
             "Compilation ignored (in the no-byte-compile list): %s" el-file)
           t)
@@ -1054,19 +1173,21 @@ EL-FILE, FEATURE, NOERROR, and NOSUFFIX are the same arguments as
          (t
           (compile-angel--debug-message "COMPILATION ARGS: %s | %s"
                                         el-file feature-symbol)
-          (puthash el-file t compile-angel--list-compiled-files)
+          (puthash el-file-truename t compile-angel--list-processed-files)
           (when feature-symbol
             (puthash feature-symbol t compile-angel--list-compiled-features))
 
           (let ((compile-angel--legacy-currently-compiling
-                 (cons el-file compile-angel--legacy-currently-compiling)))
+                 (cons el-file-truename compile-angel--legacy-currently-compiling)))
             (unwind-protect
                 (progn
-                  (puthash el-file t compile-angel--currently-compiling-files)
-                  (puthash feature-symbol t compile-angel--currently-compiling-features)
+                  (when compile-angel--currently-compiling-use-hash
+                    (puthash el-file-truename t compile-angel--currently-compiling-files)
+                    (puthash feature-symbol t compile-angel--currently-compiling-features))
                   (compile-angel--compile-elisp el-file noerror))
-              (remhash el-file compile-angel--currently-compiling-files)
-              (remhash feature-symbol compile-angel--currently-compiling-features)))))))))
+              (when compile-angel--currently-compiling-use-hash
+                (remhash el-file-truename compile-angel--currently-compiling-files)
+                (remhash feature-symbol compile-angel--currently-compiling-features))))))))))
 
 (defun compile-angel--advice-before-require (feature
                                              &optional filename noerror)
@@ -1105,12 +1226,16 @@ EL-FILE, NOERROR, and NOSUFFIX are the same args as `load'."
                               user-init-file))
             ;; Do not add loaded files to the list. Only features.
             (compile-angel-reload-compiled-version nil))
-        (compile-angel--entry-point (when el-file
-                                      (expand-file-name
-                                       (substitute-in-file-name el-file)))
-                                    nil  ; Feature
-                                    nosuffix
-                                    noerror))
+        (compile-angel--entry-point
+         ;; Only expand if it looks like a path, not a library name
+         ;; TODO Find a more standard way to detect a path
+         (if (or (file-name-absolute-p el-file)
+                 (string-match-p "/" el-file))
+             (expand-file-name (substitute-in-file-name el-file))
+           el-file)
+         nil  ; Feature
+         nosuffix
+         noerror))
     (compile-angel--debug-message
       (concat "ISSUE: Wrong type passed to "
               "compile-angel--advice-before-require %s (%s)")
@@ -1312,8 +1437,8 @@ be JIT compiled."
 
 (defun compile-angel--report-is-native-compiled (el-file)
   "Return non-nil if EL-FILE is natively compiled."
-  (when-let* ((el-file (compile-angel--normalize-el-file el-file)))
-    (when (and (not (gethash el-file
+  (when-let* ((el-file-truename (file-truename el-file)))
+    (when (and (not (gethash el-file-truename
                              compile-angel--no-byte-compile-files-list))
                (not (compile-angel--el-file-excluded-p el-file)))
       (if (not (compile-angel--elisp-native-compiled-p el-file))
@@ -1330,7 +1455,7 @@ be JIT compiled."
       (when-let* ((fname (compile-angel--normalize-el-file (car entry))))
         (compile-angel--debug-message
           "compile-angel--get-list-non-native-compiled: ADD: %s" fname)
-        (puthash fname t list-el-files)))
+        (puthash (file-truename fname) t list-el-files)))
 
     ;; Collect .el files from features
     (dolist (feature features)
@@ -1344,7 +1469,7 @@ be JIT compiled."
               (compile-angel--debug-message
                 "compile-angel--get-list-non-native-compiled: ADD: %s"
                 feature-el-file)
-              (puthash feature-el-file t list-el-files))))))
+              (puthash (file-truename feature-el-file) t list-el-files))))))
 
     ;; Check each file and build result
     (maphash
@@ -1373,7 +1498,7 @@ the corresponding .elc or .eln filenames."
     (let* ((file-origin el-file)
            (eln-file (or eln-file
                          (and (fboundp 'comp-el-to-eln-filename)
-                              (funcall 'comp-el-to-eln-filename el-file))))
+                              (comp-el-to-eln-filename el-file))))
            (elc-file (or elc-file
                          (funcall (if (bound-and-true-p
                                        byte-compile-dest-file-function)
@@ -1394,6 +1519,7 @@ the corresponding .elc or .eln filenames."
 
 (defun compile-angel-report ()
   "Create a buffer listing all Elisp files that are not native compiled."
+  (interactive)
   (with-current-buffer (get-buffer-create "*compile-angel:report*")
     (let ((inhibit-read-only t))
       (read-only-mode 1)
@@ -1429,8 +1555,13 @@ the corresponding .elc or .eln filenames."
       (progn
         ;; Init
         (compile-angel--init)
-        ;; Problematic files
-        (compile-angel--entry-point nil 'dash)
+        ;; Pre-compiling 'dash' serves as a workaround to prevent Emacs 31 from
+        ;; hanging during a fresh build. Implicitly loading 'dash' source
+        ;; (triggered by dependencies like 'elisp-refs') when byte-compilation
+        ;; is enabled can cause Emacs to hang and require restarting. This step
+        ;; ensures 'dash' is compiled explicitly if present.
+        (when compile-angel-enable-byte-compile
+          (compile-angel--entry-point nil 'dash))
         ;; Compile features
         (when compile-angel-on-load-compile-features
           (compile-angel--compile-loaded-features))
